{***************************************************************}
{              XJustOne Component for Delphi 32                 }
{ Version:     2.6                                              }
{ Author:      Gennady Makeev (makeev@cacr.ioc.ac.ru)           }
{ Improvement: Aleksey Kuznetsov (info@utilmind.com)            }
{ WWW:         http://www.utilmind.com                          }
{***************************************************************}
{ This component let start only one copy of your application.   }
{***************************************************************}
{ PROPERTIES:                                                   }
{   SwitchToPrevious: Boolean - Activate the previous instance  }
{                               if exist                        }
{ EVENTS:                                                       }
{   AlreadyExist - occurs if first instance of this application }
{                  is already loaded                            }
{***************************************************************}

unit JustOne;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms;

type
  TXJustOne = class(TComponent)
  private
    FSwitchToPrevious: Boolean;
    FAlreadyExist: TNotifyEvent;
    Mutex: hWnd;
    IsCreated: Boolean;
  public
    procedure Loaded; override;
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property SwitchToPrevious: Boolean read FSwitchToPrevious write FSwitchToPrevious;
    property AlreadyExist: TNotifyEvent read FAlreadyExist write FAlreadyExist;
  end;

procedure Register;

implementation

procedure TXJustOne.Loaded;
var
  PrevInstHandle: THandle;
  AppTitle: Array[0..$100] of Char;
begin
  inherited Loaded;
  
  StrPCopy(AppTitle, Application.Title);
  IsCreated := False;
  
  Mutex := CreateMutex(nil, False, AppTitle);
  if (GetLastError = ERROR_ALREADY_EXISTS) or
     (Mutex = 0) then
  begin
   if Assigned(FAlreadyExist) then FAlreadyExist(Self);

   if FSwitchToPrevious then
    begin
     SetWindowText(Application.Handle, '');

     PrevInstHandle := FindWindow(nil, AppTitle);
     if PrevInstHandle <> 0 then
      if IsIconic(PrevInstHandle) then
       ShowWindow(PrevInstHandle, SW_RESTORE)
      else
       BringWindowToTop(PrevInstHandle);
      SetForegroundWindow(PrevInstHandle);
    end;

   Application.ShowMainForm := False;
   Application.Terminate;
  end;
  IsCreated := True;
end;

constructor TXJustOne.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FSwitchToPrevious := True;
end;

destructor TXJustOne.Destroy;
begin
  if IsCreated then
   CloseHandle(Mutex);
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('UtilMind', [TXJustOne]);
end;

end.
